var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
export class Uint8ArrayStack {
    constructor() {
        this.stack = new Uint8Array([]);
    }
    get length() {
        return this.stack.byteLength;
    }
    /**
     * 入栈数据
     * @param uint8Array:Uint8Array
     * */
    push(uint8Array) {
        const uint8Arrays = [this.stack, uint8Array];
        const size = uint8Arrays.reduce((total, next) => total + next.byteLength, 0);
        const result = new Uint8Array(size);
        let offset = 0;
        for (let uint8Array of uint8Arrays) {
            result.set(uint8Array, offset);
            offset += uint8Array.byteLength;
        }
        this.stack = result;
    }
    /**
     * 出栈数据
     * @param len:number
     * */
    readBytes(len) {
        if (len) {
            const outBytes = this.stack.subarray(0, len);
            this.stack = this.stack.subarray(len);
            return outBytes;
        }
        else {
            return null;
        }
    }
}
/**
 * 加解密工具
 * */
export class CryptoUtil {
}
CryptoUtil.ITER = 10000;
CryptoUtil.DECRYPT_BLOCK_SIZE = 16 * 1024 * 64 * 2;
CryptoUtil.AES_BLOCK_SIZE = 16;
CryptoUtil.getKeyIv = (content, password) => {
    const salt = content.slice(8, 16);
    return CryptoUtil.getKeyAndIv(password, salt, 'decrypt');
};
CryptoUtil.decrypt = (content, key, iv) => {
    return window.crypto.subtle.decrypt({ name: "AES-CBC", iv }, key, content);
};
CryptoUtil.chunkDecrypt = (chunk, key, iv) => __awaiter(void 0, void 0, void 0, function* () {
    const nextIv = chunk.subarray(chunk.byteLength - CryptoUtil.AES_BLOCK_SIZE);
    const padding = yield crypto.subtle.encrypt({
        name: 'AES-CBC',
        iv: nextIv
    }, key, new Uint8Array(0));
    const uint8ArrayStack = new Uint8ArrayStack();
    uint8ArrayStack.push(chunk);
    uint8ArrayStack.push(new Uint8Array(padding));
    const paddedData = uint8ArrayStack.readBytes(uint8ArrayStack.length);
    const content = yield CryptoUtil.decrypt(paddedData, key, iv);
    return { chunkPlain: new Uint8Array(content), nextIv };
});
CryptoUtil.getKeyAndIv = (password, salt, keyUsages = 'encrypt') => __awaiter(void 0, void 0, void 0, function* () {
    const passphraseBytes = new Uint8Array(32);
    const passwordBytes = new TextEncoder().encode(password).slice(0, 32);
    passphraseBytes.set(passwordBytes);
    const passphraseKey = yield window.crypto.subtle.importKey('raw', passphraseBytes, { name: 'PBKDF2' }, false, ['deriveBits']);
    let pbkdf2Bytes = yield window.crypto.subtle.deriveBits({
        name: 'PBKDF2',
        salt: salt,
        iterations: CryptoUtil.ITER,
        hash: 'SHA-256'
    }, passphraseKey, 384);
    pbkdf2Bytes = new Uint8Array(pbkdf2Bytes);
    const keyBytes = pbkdf2Bytes.slice(0, 32);
    const ivBytes = pbkdf2Bytes.slice(32);
    const key = yield window.crypto.subtle.importKey('raw', keyBytes, {
        name: 'AES-CBC',
        length: 256
    }, false, ['encrypt', 'decrypt']);
    return {
        key,
        iv: ivBytes
    };
});
