/**
 * 文件名：FileSaveRegister
 * 作　者：Bryce.Li
 * 创　建：2021/6/16 11:28
 * 描　述：Bryce.Li
 * */
export default class FileSaveRegister {
    private options;
    private aborterMap;
    private progressMap;
    private completeMap;
    private errorMap;
    private messagePort;
    private active;
    constructor(scope?: string, options?: {
        confirmMessage?: string;
        onMessage?: (data: any) => void;
    });
    /**
     *  下载文件
     *  @param {string} filename 文件名
     *  @param {string | {name:string,url:string}[]} url
     *  @param {number} size 文件总大小 byte
     *  @param options
     * */
    download(filename: string, url: string | {
        path: string;
        url: string;
    }[], size?: number, options?: Partial<{
        password: string;
        cancel: (canceler: any) => void;
        onProgress: (loaded: number, size: number) => void;
        onComplete: () => void;
        onError: (message: string) => void;
    }>): void;
    /**
     * 终止下载
     * @param {string} message
     * @param {string} uuid
     * @return {boolean} 是否终止成功
     * */
    abort(message: string, uuid?: string): boolean;
    /**
     *  MessageChannel 监听事件
     *  @param {MessageEvent} event
     * */
    private _getMessage;
    /**
     * 连接 stream 数据流
     * @param {string} uniqLink:string
     * @param {number} uuid:number
     * @param {ReadableStream} readableStream
     * @param {string} password
     * @param zip
     * */
    private _pipStream;
    /**
     * 增强 Fetch，具有中断及解密功能
     * _fetch
     * @param {string} url
     * @param {string} password
     * @param {string} uuid
     * @param options
     * @returns {Promise}
     * */
    private _fetch;
    /**
     * _fetchFile
     * 单文件下载
     * @param {string} url
     * @param {string} uniqLink
     * @param {string} uuid
     * @param {string} password
     * */
    private _fetchFile;
    /**
     * _batchFetchFile
     * @param {{name:string,url:string}[]} fileList
     * @param {string} uniqLink
     * @param {string} uuid
     * @param password
     * */
    private _batchFetchFile;
}
